
#include "w_inc.h"

OEM_que wpoc_que_handler=NULL;
typedef struct{
	unsigned char type;
	unsigned char *data;
	unsigned int datasize;
}WPOC_QUE_MSG_DEF;
typedef void (*CALL_FUNC_ENTRY)(unsigned char *data, int datalen);
typedef struct{
	unsigned char type;
	CALL_FUNC_ENTRY handler;
}WPOC_EVENT_LIST;
//notes
static void call_func_note_lineoff(unsigned char *data, int datalen);
//cmds
static void call_func_cmds(unsigned char *data, int datalen);
//internal
static void call_func_login_req(unsigned char *data, int datalen);
static void call_func_query_req(unsigned char *data, int datalen);
static void call_func_ping_req(unsigned char *data, int datalen);
//extern
static void call_func_login_ack(unsigned char *data, int datalen);
static void call_func_times(unsigned char *data, int datalen);
static const WPOC_EVENT_LIST wpoc_event_entries[]={
	{WPOC_QUE_DATA_CMDS, call_func_cmds},
	{WPOC_QUE_DATA_TX_LINEOFF, call_func_note_lineoff},
	{WPOC_QUE_DATA_TX_LOGIN, call_func_login_req},
	{WPOC_QUE_DATA_TX_QUERY_GROUP, call_func_query_req},
	{WPOC_QUE_DATA_TX_PING, call_func_ping_req},
	{WPOC_QUE_DATA_RX_LOGINACK, call_func_login_ack},
	{WPOC_QUE_DATA_RX_PONG_TIMEACK, call_func_times}
};
int wpoc_que_task(void *param){
	char buf[128];
	int ret;
	WPOC_EVENT_LIST *entry;
	WPOC_QUE_MSG_DEF ack;
	ret=OEM_sys_queue_new(&wpoc_que_handler,sizeof(WPOC_QUE_MSG_DEF),50);
	if(ret!=0 || wpoc_que_handler==NULL){
		wlog_error("queue new fail");
		return 0;
	}
	wlog_info("que task start");
	for(;;){
		if(OEM_sys_queue_wait(wpoc_que_handler, &ack, sizeof(WPOC_QUE_MSG_DEF))!=0) continue;
		if(wpoc_app.usrMsgRx==NULL) continue; 
		entry=wpoc_event_entries;
		while(entry->handler){
			if(entry->type==ack.type){
				entry->handler(ack.data, ack.datasize);
				break;
			}
			entry++;
		}
		if(ack.data!=NULL) OEM_sys_free(ack.data);
	}
}

int wpoc_que_data_send(unsigned char datatype, unsigned char *data, unsigned int datalen){
	WPOC_QUE_MSG_DEF send={
		.data=NULL,
		.datasize=0
	};
	if(wpoc_que_handler==NULL) return -1;
	if(datalen!=0){
		if(data==NULL) return -2;
		send.data=(unsigned char *)OEM_sys_malloc(datalen+1);
		if(send.data==NULL) return -3;
		send.datasize=datalen;
		memcpy(send.data, data, datalen);
		send.data[datalen]=0;
	}
	send.type=datatype;
	return OEM_sys_queue_send(wpoc_que_handler, &send, sizeof(WPOC_QUE_MSG_DEF));
}

//internal
static void call_func_login_req(unsigned char *data, int datalen){
	WPOC_CMD_DATA_DEF *tdata=(WPOC_CMD_DATA_DEF *)data;
	OEM_sock *sock=wpoc_get_tcp_sock();
	WPOC_DEF *info=(WPOC_DEF *)tdata->param1;
	wpoc_login_req(sock, info->psn, info->pass,info->hardid,info->platform);
}
static void call_func_query_req(unsigned char *data, int datalen){
	OEM_sock *sock=wpoc_get_tcp_sock();
	wpoc_query_group_req(sock);
}
static void call_func_ping_req(unsigned char *data, int datalen){
	WPOC_CMD_DATA_DEF *tdata=(WPOC_CMD_DATA_DEF *)data;
	OEM_sock *sock=wpoc_get_tcp_sock();
	wpoc_ping_req(sock,tdata->data1);
}
//extern
static void call_func_login_ack(unsigned char *data, int datalen){
	WPOC_MSG_TR_AT_FUN at_fun=wpoc_app.usrMsgRx->wpoc_msg_rx_at_fun;
	WPOC_MSG_RX_EVENT_FUN cb_fun=wpoc_app.usrMsgRx->wpoc_msg_rx_event_fun;
	WPOC_LOGINACK_DEF *uinfo=(WPOC_LOGINACK_DEF *)data;
	if(at_fun!=NULL){
		char buf[64];
		at_fun("+WPTT:LOGIN=1\r\n");
		snprintf(buf, sizeof(buf), "+WPTT:ME=%08x,",uinfo->user_id);
		strcat_hexstr_string(buf, uinfo->user_name,true);
		at_fun(buf);
		snprintf(buf, sizeof(buf), "+WPTT:BE=%08x,%04x,",uinfo->be_group_id,uinfo->user_num_in_group);
		strcat_hexstr_string(buf, uinfo->be_group_name,true);
		at_fun(buf);
	}else if(cb_fun!=NULL){
		cb_fun(WPOC_RX_EVENT_LOGIN, data, datalen);
	}
}
static void call_func_times(unsigned char *data, int datalen){
	WPOC_MSG_TR_AT_FUN at_fun=wpoc_app.usrMsgRx->wpoc_msg_rx_at_fun;
	WPOC_MSG_RX_EVENT_FUN cb_fun=wpoc_app.usrMsgRx->wpoc_msg_rx_event_fun;
	char info[40];
	if(at_fun!=NULL){
		snprintf(info, sizeof(info), "+WPTT:TIME=%s\r\n", (char *)data);
		at_fun(info);
	}else if(cb_fun!=NULL){
		cb_fun(WPOC_RX_EVENT_TIMES, data, datalen);
	}
}

//cmds
static void call_func_cmds(unsigned char *data, int datalen){
	const char *func_cmds_mark="AT+WPTT:";
	static char msg[64],ch;
	static short msgIndex=0;
	char *info;
	int i=0;
	
	for(i=0;i<datalen;i++){
		if(msgIndex>=sizeof(msg)) msgIndex=0;
		msg[msgIndex++]=data[i];
		if(ch!='\r' || data[i]!='\n'){
			ch=data[i];
			continue;
		}
		if(msgIndex<=2){
			msgIndex=0;
			continue;
		}
		msg[msgIndex-2]=0;
		wlog_info("IN:%s", data);
		//process
		if(!wpoc_msg_cmp(msg, func_cmds_mark)){
			msgIndex=0;
			continue;
		}
		info=&msg[strlen(func_cmds_mark)];
		if(wpoc_msg_cmp(info, "REQTIME")) w_poc_cmd_que_tx_ping(true);
		msgIndex=0;
	}
}
//notes
static void call_func_note_lineoff(unsigned char *data, int datalen){
	WPOC_MSG_TR_AT_FUN at_fun=wpoc_app.usrMsgRx->wpoc_msg_rx_at_fun;
	WPOC_MSG_RX_EVENT_FUN cb_fun=wpoc_app.usrMsgRx->wpoc_msg_rx_event_fun;
	if(at_fun!=NULL){
		at_fun("+WPTT:LINEOFF\r\n");
	}else if(cb_fun!=NULL){
		cb_fun(WPOC_RX_EVENT_LINEOFF, NULL, 0);
	}
}
////TX FUNS
int wpoc_msg_at_tx(char *msg){
	wpoc_que_data_send(WPOC_QUE_DATA_CMDS, msg, strlen(msg));
}
int wpoc_msg_event_tx(WPOC_EVENT_TX event, unsigned char *data, int datalen){
	if(wpoc_app.poc_tcp==0) return -1;
	if(wpoc_app.poc_login==0) return -2;
	WPOC_REQ_GROUP_DEF *groupQuery;
	switch(event){
		case WPOC_TX_EVENT_REQGOUPS:
			groupQuery=(WPOC_REQ_GROUP_DEF *)data;
			if(groupQuery==NULL) return -3;
			w_poc_cmd_que_tx_ping(groupQuery->startIndex, groupQuery->totalNum);
			break;
		case WPOC_TX_EVENT_REQTIME:
			w_poc_cmd_que_tx_ping(true);
			break;
	}
	return 0;
}