
#include "w_inc.h"
#define TCP_FORMAT_INDICATOR 0x6F
typedef enum{
	U_LOGIN=1,
	U_QUERY_GROUP,
	U_JOIN_GROUP,
	U_REQUEST_MIC,
	U_RELEASE_MIC,
	U_QUERY_MEMBER,
	U_BUILD_TEMP_CALL,
	U_LEAVE_TEMP_CALL,
	U_PING,
	U_LOGOUT,
	U_UPLOAD_MAX
}W_POC_UPLOAD_CMD_ENUM;

typedef enum{
	D_LOGIN_ACK = 1,
	D_QUERY_GROUP_ACK,
	D_JOIN_GROUP_ACK,
	D_REQUEST_MIC_ACK,
	D_RELEASE_MIC_ACK,
	D_QUERY_MEMBER_ACK,
	D_LOST_MIC_ACK,
	D_CALLER_INFO_ACK,
	D_BUILD_TEMP_CALL_ACK,
	D_LEAVE_TEMP_CALL_ACK,
	D_PONG,
	D_LOGOUT_ACK,
	D_DOWNLOAD_MAX
}W_POC_DOWNLOAD_CMD_ENUM;

unsigned char *w_packing_buf(int *outsize, unsigned char cmd, unsigned char *usrbuf, int usrsize);
unsigned char* codePackIndicator(unsigned char *msg, int msglen,int *outlen);
int wpoc_login_req(OEM_sock *handle, unsigned int psn, char *pass,char *hardid,char *platform){
	wlog_info("try login req");
	Login _login=LOGIN__INIT;
	_login.psn=psn;
	_login.pass=pass;
	_login.hardid=hardid;
	_login.platform=platform;
	int rlen=login__get_packed_size(&_login);
	unsigned char *pb=(unsigned char *)malloc(rlen+1);
	if(pb==NULL) return -1;
	rlen=login__pack(&_login, pb+1);
	int outsize;
	unsigned char *out=w_packing_buf(&outsize, U_LOGIN, pb, rlen+1);
	free(pb);
	if(out==NULL) return -2;
	
	rlen=OEM_sys_sock_send_data(handle, out, outsize);
	return rlen;
}
int wpoc_query_group_req(OEM_sock *handle){
	wlog_info("try query group");
	QueryGroup _querygroup=QUERY_GROUP__INIT;
	_querygroup.detail=QUERY_GROUP__DETAIL_LEVEL__ALL_INFO;
	_querygroup.has_max_groups=true;
	wpoc_app.groupList.usrSetMaxGroupNum=wpoc.maxGroupNum;
	_querygroup.max_groups=wpoc_app.groupList.usrSetMaxGroupNum;
	int rlen=query_group__get_packed_size(&_querygroup);
	unsigned char *pb=(unsigned char *)malloc(rlen+1);
	if(pb==NULL) return -1;
	rlen=query_group__pack(&_querygroup, pb+1);
	int outsize;
	unsigned char *out=w_packing_buf(&outsize, U_QUERY_GROUP, pb, rlen+1);
	free(pb);
	if(out==NULL) return -2;
	
	rlen=OEM_sys_sock_send_data(handle, out, outsize);
	return rlen;
}
int wpoc_ping_req(OEM_sock *handle,bool need_time_ack){
	wlog_info("try ping req:%s", need_time_ack==true?"with time":"no time");
	Ping _ping=PING__INIT;
	if(need_time_ack==true){
		_ping.has_datereq=1;
		_ping.has_timereq=1;
		_ping.datereq=1;
		_ping.timereq=1;
	}
	int rlen=ping__get_packed_size(&_ping);
	unsigned char *pb=(unsigned char *)malloc(rlen+1);
	if(pb==NULL) return -1;
	rlen=ping__pack(&_ping, pb+1);
	int outsize;
	unsigned char *out=w_packing_buf(&outsize, U_PING, pb, rlen+1);
	free(pb);
	if(out==NULL) return -2;
	
	rlen=OEM_sys_sock_send_data(handle, out, outsize);
	return rlen;
}
void bufDataEncrypt(unsigned char code, unsigned char *msg, int len){
	for(int i=0;i<len;i++) msg[i] ^= (code | TCP_FORMAT_INDICATOR);
}
unsigned char *w_packing_buf(int *outsize, unsigned char upload_cmd, unsigned char *usrbuf, int usrsize){
	usrbuf[0]=upload_cmd;
	bufDataEncrypt(upload_cmd, usrbuf+1, usrsize-1);
	return codePackIndicator(usrbuf, usrsize,outsize);
}
/*
TCP_FORMAT_INDICATOR -->TCP_FORMAT_INDICATOR+1 0x02
TCP_FORMAT_INDICATOR+1 --> TCP_FORMAT_INDICATOR+1 0x01

01 9f 77 6f 70 01 70 02 88
[6f] 01 9f 77 70 02 70 01 01 70 01 02 88 [6f] 
*/
unsigned char* codePackIndicator(unsigned char *msg, int msglen,int *outlen){
	int totalsize=2,i;
	unsigned char ch=0;
	for(i=0;i<msglen;i++){
		if(msg[i]==TCP_FORMAT_INDICATOR || msg[i]==TCP_FORMAT_INDICATOR+1) totalsize ++;
		totalsize++;
	}
	unsigned char *nbuf=(unsigned char *)malloc(totalsize);
	if(nbuf==NULL) return NULL;
	unsigned char *ptr=nbuf;
	*ptr++=TCP_FORMAT_INDICATOR;
	for(i=0;i<msglen;i++){
		if(msg[i]==TCP_FORMAT_INDICATOR){
			*ptr++=TCP_FORMAT_INDICATOR+1;
			*ptr++=0x02;
		}else if(msg[i]==TCP_FORMAT_INDICATOR+1){
			*ptr++=TCP_FORMAT_INDICATOR+1;
			*ptr++=0x01;
		}else *ptr++=msg[i];
		ch=msg[i];
	}
	*ptr=TCP_FORMAT_INDICATOR;
	*outlen=totalsize;
	return nbuf;
}

/*
TCP_FORMAT_INDICATOR+1 0x01 --> TCP_FORMAT_INDICATOR+1
TCP_FORMAT_INDICATOR+1 0x02 --> TCP_FORMAT_INDICATOR
[6f] 01 9f 77 70 02 70 01 01 70 01 02 88 [6f] 
01 9f 77 6f 70 01 70 02 88
*/
int decoderPackIndicator(unsigned char *msg, int msglen){
	int i,k=0;
	unsigned char *ptr=msg;
	for(i=1;i<msglen-1;i++){
		if(msg[i]==TCP_FORMAT_INDICATOR+1){
			if(msg[i+1]==0x01){
				ptr[k++]=TCP_FORMAT_INDICATOR+1;
				i++;
				continue;
			}else if(msg[i+1]==0x02){
				ptr[k++]=TCP_FORMAT_INDICATOR;
				i++;
				continue;
			}
		}
		ptr[k++]=msg[i];
	}
	return k;
}

typedef void (*WPOC_MSG_HANDLER)(const ProtobufCMessage *msg);
typedef struct{
	unsigned char cmd;
	const ProtobufCMessageDescriptor *msg_descriptor;
	WPOC_MSG_HANDLER handler;
}W_POC_MSG_ENTRY_DEF;
static void d_login_ack(const ProtobufCMessage *msg);
static void d_query_group_ack(const ProtobufCMessage *msg);
static void d_join_group_ack(const ProtobufCMessage *msg);
static void d_request_mic_ack(const ProtobufCMessage *msg);
static void d_release_mic_ack(const ProtobufCMessage *msg);
static void d_query_member_ack(const ProtobufCMessage *msg);
static void d_lost_mic_ack(const ProtobufCMessage *msg);
static void d_caller_info_ack(const ProtobufCMessage *msg);
static void d_build_temp_call_ack(const ProtobufCMessage *msg);
static void d_leave_temp_call_ack(const ProtobufCMessage *msg);
static void d_pong(const ProtobufCMessage *msg);
static void d_logout_ack(const ProtobufCMessage *msg);

static const W_POC_MSG_ENTRY_DEF wpoc_download_msg_entries[]={
	{D_LOGIN_ACK, &login_ack__descriptor, d_login_ack},
	{D_QUERY_GROUP_ACK, &query_group_ack__descriptor, d_query_group_ack},
	{D_JOIN_GROUP_ACK, &join_group_ack__descriptor, d_join_group_ack},
	{D_REQUEST_MIC_ACK, &request_mic_ack__descriptor, d_request_mic_ack},
	{D_RELEASE_MIC_ACK, &release_mic_ack__descriptor, d_release_mic_ack},
	{D_QUERY_MEMBER_ACK, &query_member_ack__descriptor, d_query_member_ack},
	{D_LOST_MIC_ACK, &lost_mic__descriptor, d_lost_mic_ack},
	{D_CALLER_INFO_ACK, &take_mic_member__descriptor, d_caller_info_ack},
	{D_BUILD_TEMP_CALL_ACK, &build_temp_call_ack__descriptor, d_build_temp_call_ack},
	{D_LEAVE_TEMP_CALL_ACK, &leave_temp_call_ack__descriptor, d_leave_temp_call_ack},
	{D_PONG, &pong__descriptor, d_pong},
	{D_LOGOUT_ACK, &logout_ack__descriptor, d_logout_ack}
};

void w_unpacking_buf(unsigned char *usrbuf, int usrsize){
	int ulen=decoderPackIndicator(usrbuf, usrsize);
	unsigned char cmd=usrbuf[0];
	bufDataEncrypt(cmd, usrbuf+1, ulen-1);
	unsigned char *info=usrbuf+1;
	ulen--;
	
	if(cmd>=D_DOWNLOAD_MAX){
		wlog_warn("unknown download cmd:%02x", cmd);
		return;
	}
	W_POC_MSG_ENTRY_DEF *entry=wpoc_download_msg_entries;
	ProtobufCMessage *msg;
	while(entry->handler){
		if(entry->cmd==cmd){
			msg=protobuf_c_message_unpack(entry->msg_descriptor, NULL, ulen, info);
			if(msg==NULL){
				wlog_error("msg unpack error:0x%02x", cmd);
				break;
			}
			entry->handler(msg);
			protobuf_c_message_free_unpacked (msg, NULL);
			break;
		}
		entry++;
	}
}

static void d_login_ack(const ProtobufCMessage *msg){
	LoginAck *ack=(LoginAck *)msg;
	if(ack->result!=0){
		wlog_warn("login fail:%d",ack->result);
		return;
	}
	wlog_warn("login ok:port=%d",ack->confi->talk_port);
	wpoc_app.poc_login=1;
	wpoc_app.poc_firstLogin=1;
	wpoc_app.poc_voice_port=ack->confi->talk_port;
	WPOC_LOGINACK_DEF uinfo;
	uinfo.user_id=ack->usr->id;
	snprintf(uinfo.user_name, sizeof(uinfo.user_name), "%s", ack->usr->name);
	uinfo.be_group_id=ack->currg_group->id;
	snprintf(uinfo.be_group_name, sizeof(uinfo.be_group_name), "%s", ack->currg_group->name);
	uinfo.user_num_in_group=ack->currg_group->umembers;
	wpoc_que_data_send(WPOC_QUE_DATA_RX_LOGINACK, &uinfo, sizeof(WPOC_LOGINACK_DEF));
	wpoc_que_data_send(WPOC_QUE_DATA_TX_QUERY_GROUP, NULL, 0);
}
static void d_query_group_ack(const ProtobufCMessage *msg){
	QueryGroupAck *ack=(QueryGroupAck *)msg;
	if(ack->result!=0){
		wlog_warn("query group fail:%d", ack->result);
		return;
	}
	wlog_info("query group ok:%u", ack->n_groups);
	int i;
	unsigned int num;
	GROUP_INFO_LIST *gInfo=&wpoc_app.groupList;
	if(gInfo->usrSetMaxGroupNum==0) gInfo->usrSetMaxGroupNum=100;
	if(gInfo->groupInfo==NULL || gInfo->maxGroupNum!=gInfo->usrSetMaxGroupNum){
		if(gInfo->groupInfo!=NULL){
			for(i=0;i<gInfo->maxGroupNum;i++) OEM_sys_free(gInfo->groupInfo[i]);
			OEM_sys_free(gInfo->groupInfo);
			//try to new
			num=gInfo->usrSetMaxGroupNum;
			gInfo->groupInfo=(GROUP_INFO_DEF **)OEM_sys_malloc(num*sizeof(GROUP_INFO_DEF*));
			if(gInfo->groupInfo==NULL){
				wlog_error("groupInfo new fail");
				return;
			}
			for(i=0;i<num;i++){
				gInfo->groupInfo[i]=(GROUP_INFO_DEF *)OEM_sys_malloc(sizeof(GROUP_INFO_DEF));
				if(gInfo->groupInfo[i]==NULL){
					wlog_error("groupInfo new no enough:%d,%d", i, num);
					for(num=0;i<i;i++) OEM_sys_free(gInfo->groupInfo[i]);
					OEM_sys_free(gInfo->groupInfo);
					gInfo->groupInfo=NULL;
					return;
				}
			}
		}
		//start to save
		gInfo->maxGroupNum=gInfo->usrSetMaxGroupNum;
		num=(ack->n_groups>gInfo->maxGroupNum?gInfo->maxGroupNum:ack->n_groups);
		for(i=0;i<num;i++){
			wlog_info("group<%d>id=%08x,mem=%d,name=%s", i,ack->groups[i]->id, ack->groups[i]->umembers,ack->groups[i]->name);
			gInfo->groupInfo[i]->id=ack->groups[i]->id;
			gInfo->groupInfo[i]->mem_num=ack->groups[i]->umembers;
			snprintf(gInfo->groupInfo[i]->name,sizeof(gInfo->groupInfo[i]->name), "%s", ack->groups[i]->name);
		}
		gInfo->validGroupNum=num;
	}
}
static void d_join_group_ack(const ProtobufCMessage *msg){
	JoinGroupAck *ack=(JoinGroupAck *)msg;
}
static void d_request_mic_ack(const ProtobufCMessage *msg){
	RequestMicAck *ack=(RequestMicAck *)msg;
}
static void d_release_mic_ack(const ProtobufCMessage *msg){
	ReleaseMicAck *ack=(ReleaseMicAck *)msg;
}
static void d_query_member_ack(const ProtobufCMessage *msg){
	QueryMemberAck *ack=(QueryMemberAck *)msg;
}
static void d_lost_mic_ack(const ProtobufCMessage *msg){
	LostMic *ack=(LostMic *)msg;
}
static void d_caller_info_ack(const ProtobufCMessage *msg){
	TakeMicMember *ack=(TakeMicMember *)msg;
}
static void d_build_temp_call_ack(const ProtobufCMessage *msg){
	BuildTempCallAck *ack=(BuildTempCallAck *)msg;
}
static void d_leave_temp_call_ack(const ProtobufCMessage *msg){
	LeaveTempCallAck *ack=(LeaveTempCallAck *)msg;
}
static void d_pong(const ProtobufCMessage *msg){
	Pong *ack=(Pong *)msg;
	wlog_info("pong");
	if(ack->date!=NULL && ack->time!=NULL){
		char buf[30];
		snprintf(buf, sizeof(buf), "%s %s", ack->date, ack->time);
		wpoc_que_data_send(WPOC_QUE_DATA_RX_PONG_TIMEACK, buf, strlen(buf));
	}
}
static void d_logout_ack(const ProtobufCMessage *msg){
	LogoutAck *ack=(LogoutAck *)msg;
}

void w_poc_cmd_que_notes(void){
	WPOC_CMD_DATA_DEF usrdata={
	};
	wpoc_que_data_send(WPOC_QUE_DATA_TX_LINEOFF, &usrdata, sizeof(WPOC_CMD_DATA_DEF));
}
void w_poc_cmd_que_tx_login(void *param1){
	WPOC_CMD_DATA_DEF usrdata={
		.param1=param1
	};
	wpoc_que_data_send(WPOC_QUE_DATA_TX_LOGIN, &usrdata, sizeof(WPOC_CMD_DATA_DEF));
}
void w_poc_cmd_que_tx_query_group(void){
	wpoc_que_data_send(WPOC_QUE_DATA_TX_QUERY_GROUP, NULL, 0);
}
void w_poc_cmd_que_tx_ping(unsigned int value){
	WPOC_CMD_DATA_DEF usrdata={
		.data1=value
	};
	wpoc_que_data_send(WPOC_QUE_DATA_TX_PING, &usrdata, sizeof(WPOC_CMD_DATA_DEF));
}